#include "Arduino_BMI270_BMM150.h"
#include <HardwareBLESerial.h>

HardwareBLESerial &bleSerial = HardwareBLESerial::getInstance();

char c;
String command;
String dataString;
const int numberTrials = 100;
int trialNumber = 1;
int sampleNumber;
const int numberSamples = 125;
float aX, aY, aZ;
float imuData[numberSamples][4];

void setup() {
  //Serial.begin(9600);
  //while (!Serial);

  //Serial.print("Initializing IMU...");
  if (!IMU.begin()) {
    //Serial.println("initialization failed!");
    return;
  }
  //Serial.println("initialization done.");

  /*Serial.print("Accelerometer sample rate = ");
  Serial.print(IMU.accelerationSampleRate());
  Serial.println(" Hz");
  Serial.print("Gyroscope sample rate = ");
  Serial.print(IMU.gyroscopeSampleRate());
  Serial.println(" Hz");*/

  if (!bleSerial.beginAndSetupBLE("IMU BMI270")) {
    while (true) {
      //Serial.println("failed to initialize HardwareBLESerial!");
      delay(1000);
    }
  }

  // wait for a central device to connect
  while (!bleSerial);
  //Serial.println("HardwareBLESerial central device connected!");
}

void loop() {
  // this must be called regularly to perform BLE updates
  bleSerial.poll();
  delay(100);

  // whatever is written to BLE UART appears in the Serial Monitor
  while (bleSerial.available() > 0) {
    c = bleSerial.read();
    command += c;
    if (command == "Start Trial") {
      command = "";
      dataString = "Trial " + String(trialNumber) + " is ready. Clear terminal and press Go.\n";
      bleSerialization();
    } else if (command == "Go") {
      command = "";
      sampleNumber = 0;
      startTrial();
    } else if (command == "Redo Trial") {
      command = "";
      trialNumber--;
      dataString = "Trial " + String(trialNumber) + " will be redone. Clear terminal and press Go.\n";
      bleSerialization();
    } else if (command == "New Trial Series") {
      command = "";
      trialNumber = 1;
      dataString = "A new trial series will commence with Trial 1. Clear terminal and press Go.\n";
      bleSerialization();
    } else if (command == "Send") {
      command = "";
      sendDataPhone();
      trialNumber++;
    } else if (command.length() >= 16) {
      command = "";
    }
  }
}

void startTrial() {
  if (trialNumber > numberTrials) {
    dataString = "Trials Complete!\n";
    bleSerialization();
    //Serial.println("Trials Complete!");
    return;
  }
  if (trialNumber <= numberTrials) {
    delay(100);

    do {
      if (IMU.accelerationAvailable()) {
        IMU.readAcceleration(aX, aY, aZ);
        imuData[sampleNumber][0] = aX;
        imuData[sampleNumber][1] = aZ;
        imuData[sampleNumber][2] = aY;
        imuData[sampleNumber][3] = sqrt((aX * aX) + (aZ * aZ) + (aY * aY));
        sampleNumber++;
      }
    } while (sampleNumber != numberSamples);
  }
}

void sendDataPhone() {
  //Serial.println("Sending Trial Data to Phone.");

  dataString = "X-axis Accelerometer";
  dataString += ",";
  dataString += "Y-axis Accelerometer";
  dataString += ",";
  dataString += "Z-axis Accelerometer";
  dataString += ",";
  dataString += "Magnitude";
  dataString += "\n";

  bleSerialization();

  for (int k = 0; k < numberSamples; k++) {
    /*Serial.print(imuData[k][0], 6);
    Serial.print(",");
    Serial.print(imuData[k][1], 6);
    Serial.print(",");
    Serial.print(imuData[k][2], 6);
    Serial.print(",");
    Serial.print(imuData[k][3], 6);
    Serial.print(",");
    Serial.print(imuData[k][4], 6);
    Serial.print(",");
    Serial.println(imuData[k][5], 6);*/

    dataString = String(imuData[k][0], 6);
    dataString += ",";
    dataString += String(imuData[k][1], 6);
    dataString += ",";
    dataString += String(imuData[k][2], 6);
    dataString += ",";
    dataString += String(imuData[k][3], 6);
    dataString += "\n";

    bleSerialization();
  }
}

void bleSerialization() {
  const char *sample = dataString.c_str();
  for (size_t l = 0; l < strlen(sample); l++) {
    bleSerial.write(sample[l]);
    delay(2);
  }
}